@@ -21,6 +21,7 @@
 #include <QTextStream>
 #include <QFile>
 #include <QDir>
+#include <QSslSocket>
 #include "x2gologdebug.h"
 #include <QMessageBox>
 #include <QDateTime>
@@ -41,6 +42,7 @@ HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
     QUrl lurl ( config->brokerurl );
     if(lurl.userName().length()>0)
         config->brokerUser=lurl.userName();
+
     if(config->brokerurl.indexOf("ssh://")==0)
     {
         sshBroker=true;
@@ -53,18 +55,31 @@ HttpBrokerClient::HttpBrokerClient ( ONMainWindow* wnd, ConfigFile* cfg )
     else
     {
         sshBroker=false;
+
         http=new QHttp ( this );
-        if ( config->brokerurl.indexOf ( "https://" ) ==0 )
+
+        if ( config->brokerurl.indexOf ( "https://" ) ==0 ) {
+            if ((config->brokerCaCertFile.length() >0) && (QFile::exists(config->brokerCaCertFile))) {
+
+                sslSocket = new QSslSocket(this);
+                connect ( sslSocket, SIGNAL ( sslErrors ( const QList<QSslError>& ) ),this,
+                          SLOT ( slotSslErrors ( const QList<QSslError>& ) ) );
+                http->setSocket(sslSocket);
+                sslSocket->addCaCertificates(config->brokerCaCertFile, QSsl::Pem);
+                x2goDebug<<"Custom CA certificate file loaded into HTTPS broker client: "<<config->brokerCaCertFile;
+
+            } else {
+                connect ( http, SIGNAL ( sslErrors ( const QList<QSslError>& ) ),this,
+                          SLOT ( slotSslErrors ( const QList<QSslError>& ) ) );
+            }
             http->setHost ( lurl.host(),QHttp::ConnectionModeHttps,
                             lurl.port ( 443 ) );
-        else
+        } else {
             http->setHost ( lurl.host(),QHttp::ConnectionModeHttp,
                             lurl.port ( 80 ) );
-
+        }
         connect ( http,SIGNAL ( requestFinished ( int,bool ) ),this,
                   SLOT ( slotRequestFinished ( int,bool ) ) );
-        connect ( http,SIGNAL ( sslErrors ( const QList<QSslError>& ) ),this,
-                  SLOT ( slotSslErrors ( const QList<QSslError>& ) ) );
     }
 }
 
@@ -77,7 +92,8 @@ void HttpBrokerClient::createSshConnection()
 {
     QUrl lurl ( config->brokerurl );
     sshConnection=new SshMasterConnection (this, lurl.host(), lurl.port(22),false,
-                                           config->brokerUser, config->brokerPass,config->brokerSshKey,config->brokerAutologin, false,false);
+                                           config->brokerUser, config->brokerPass,config->brokerSshKey,config->brokerAutologin,
+					   config->brokerKrbLogin, false);
 
     connect ( sshConnection, SIGNAL ( connectionOk(QString)), this, SLOT ( slotSshConnectionOk() ) );
     connect ( sshConnection, SIGNAL ( serverAuthError ( int,QString, SshMasterConnection* ) ),this,
@@ -515,7 +531,7 @@ void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
     QSslCertificate cert;
     for ( int i=0; i<errors.count(); ++i )
     {
-        x2goDebug<<"sslError ,code:"<<errors[i].error() <<":";
+        x2goDebug<<"sslError, code:"<<errors[i].error() <<":";
         err<<errors[i].errorString();
         if ( !errors[i].certificate().isNull() )
             cert=errors[i].certificate();
@@ -527,10 +543,10 @@ void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
     fname=fname.replace(":","_");
     QUrl lurl ( config->brokerurl );
     QString homeDir=mainWindow->getHomeDirectory();
-    if ( QFile::exists ( homeDir+"/ssl/exceptions/"+
+    if ( QFile::exists ( homeDir+"/.x2go/ssl/exceptions/"+
                          lurl.host() +"/"+fname ) )
     {
-        QFile fl ( homeDir+"/ssl/exceptions/"+
+        QFile fl ( homeDir+"/.x2go/ssl/exceptions/"+
                    lurl.host() +"/"+fname );
         fl.open ( QIODevice::ReadOnly | QIODevice::Text );
         QSslCertificate mcert ( &fl );
@@ -601,14 +617,14 @@ void HttpBrokerClient::slotSslErrors ( const QList<QSslError> & errors )
     {
         x2goDebug<<"accept certificate";
         QDir dr;
-        dr.mkpath ( homeDir+"/ssl/exceptions/"+lurl.host() +"/" );
-        QFile fl ( homeDir+"/ssl/exceptions/"+
+        dr.mkpath ( homeDir+"/.x2go/ssl/exceptions/"+lurl.host() +"/" );
+        QFile fl ( homeDir+"/.x2go/ssl/exceptions/"+
                    lurl.host() +"/"+fname );
         fl.open ( QIODevice::WriteOnly | QIODevice::Text );
         QTextStream ( &fl ) <<cert.toPem();
         fl.close();
         http->ignoreSslErrors();
-        x2goDebug<<"store certificate in  "<<homeDir+"/ssl/exceptions/"+
+        x2goDebug<<"store certificate in  "<<homeDir+"/.x2go/ssl/exceptions/"+
                  lurl.host() +"/"+fname;
         requestTime.restart();
     }
@@ -628,3 +644,4 @@ QString HttpBrokerClient::getHexVal ( const QByteArray& ba )
     }
     return val.join ( ":" );
 }
+
