@@ -167,7 +167,9 @@ struct ConfigFile
     bool brokerNoAuth;
     bool brokerAutologin;
     bool brokerAutologoff;
+    bool brokerKrbLogin;
     QString brokerSshKey;
+    QString brokerCaCertFile;
     QString iniFile;
     QString server;
     QString serverIp;//Can be different from server (use for loadballancing)
@@ -214,6 +216,7 @@ struct ConfigFile
     QString proxypassword;
     QString proxyKey;
     bool proxyAutologin;
+    bool proxyKrbLogin;
 
 };
 
@@ -594,6 +597,7 @@ private:
     QList<serv> x2goServers;
     QList<Application> applications;
     QList<sshKey> cmdSshKeys;
+    bool forceToShowTrayicon; //true if --tray-icon passed in command line
 
     QPushButton* bSusp;
     QPushButton* bTerm;
@@ -892,7 +896,7 @@ private:
             SshMasterConnection::ProxyType type=SshMasterConnection::PROXYSSH,
             QString proxyserver=QString::null, quint16 proxyport=0,
             QString proxylogin=QString::null, QString proxypassword=QString::null, QString proxyKey=QString::null,
-            bool proxyAutologin=false );
+            bool proxyAutologin=false, bool proxyKrbLogin=false );
     void setProxyWinTitle();
     QRect proxyWinGeometry();
     void readApplications();
@@ -939,7 +943,7 @@ private slots:
     void showSessionStatus();
     void slotSshConnectionError ( QString message, QString lastSessionError );
     void slotSshServerAuthError ( int error, QString sshMessage, SshMasterConnection* connection );
-    void slotSshServerAuthPassphrase ( SshMasterConnection* connection );
+    void slotSshServerAuthPassphrase ( SshMasterConnection* connection, bool verificationCode );
     void slotSshUserAuthError ( QString error );
     void slotSshConnectionOk();
     void slotServSshConnectionOk(QString server);
