@@ -16,6 +16,7 @@
 ***************************************************************************/
 
 #include "sessionwidget.h"
+#include "x2goutils.h"
 #include <QBoxLayout>
 #include <QLineEdit>
 #include <QPushButton>
@@ -153,6 +154,8 @@ SessionWidget::SessionWidget ( QString id, ONMainWindow * mw,
         QIcon ( mainWindow->iconsPath ( "/16x16/file-open.png" ) ),
         QString::null,proxyBox );
     cbProxyAutologin=new QCheckBox(tr("ssh-agent or default ssh key"),proxyBox);
+    cbProxyKrbLogin=new QCheckBox(tr("Kerberos 5 (GSSAPI) authentication"),proxyBox);
+
 
     proxyLaout->addWidget(new QLabel(tr("Type:"),proxyBox),0,0,1,2);
     proxyLaout->addWidget(rbSshProxy,1,0,1,2);
@@ -170,6 +173,7 @@ SessionWidget::SessionWidget ( QString id, ONMainWindow * mw,
     proxyLaout->addWidget(proxyKey,3,4,1,1);
     proxyLaout->addWidget(pbOpenProxyKey,3,5,1,1);
     proxyLaout->addWidget(cbProxyAutologin,4,3,1,3);
+    proxyLaout->addWidget(cbProxyKrbLogin,5,3,1,3);
 
 
 #ifndef Q_WS_HILDON
@@ -266,8 +270,6 @@ SessionWidget::SessionWidget ( QString id, ONMainWindow * mw,
     connect (cbProxySameUser, SIGNAL(clicked(bool)), this, SLOT(slot_proxySameLogin()));
 
     readConfig();
-    cbKrbLogin->setChecked(false);
-    cbKrbLogin->setVisible(false);
 }
 
 
@@ -334,6 +336,7 @@ void SessionWidget::slot_proxyType()
 {
     bool isSsh=rbSshProxy->isChecked();
     cbProxyAutologin->setVisible(isSsh);
+    cbProxyKrbLogin->setVisible(isSsh);
     proxyKey->setVisible(isSsh);
     proxyKeyLabel->setVisible(isSsh);
     pbOpenProxyKey->setVisible(isSsh);
@@ -503,21 +506,22 @@ void SessionWidget::readConfig()
     sessName->setText (
         st.setting()->value (
             sessionId+"/name",
-            ( QVariant ) tr ( "New session" ) ).toString() );
+            ( QVariant ) tr ( "New session" ) ).toString().trimmed() );
     sessIcon=st.setting()->value (
                  sessionId+"/icon",
-                 ( QVariant ) ":icons/128x128/x2gosession.png" ).toString();
+                 ( QVariant ) ":icons/128x128/x2gosession.png" ).toString().trimmed();
+    sessIcon=expandHome(sessIcon);
     icon->setIcon ( QIcon ( sessIcon ) );
 
     server->setText ( st.setting()->value (
                           sessionId+"/host",
-                          ( QVariant ) QString::null ).toString() );
+                          ( QVariant ) QString::null ).toString().trimmed() );
     uname->setText ( st.setting()->value (
                          sessionId+"/user",
-                         ( QVariant ) QString::null ).toString() );
+                         ( QVariant ) QString::null ).toString().trimmed() );
     key->setText ( st.setting()->value (
                        sessionId+"/key",
-                       ( QVariant ) QString::null ).toString() );
+                       ( QVariant ) QString::null ).toString().trimmed() );
     cbAutoLogin->setChecked(st.setting()->value (
                                 sessionId+"/autologin",
                                 ( QVariant ) false ).toBool());
@@ -545,7 +549,7 @@ void SessionWidget::readConfig()
     QString prtype= st.setting()->value (
                         sessionId+"/sshproxytype",
                         "SSH"
-                    ).toString() ;
+                    ).toString().trimmed() ;
 
     if(prtype=="HTTP")
     {
@@ -559,17 +563,17 @@ void SessionWidget::readConfig()
     proxyLogin->setText(st.setting()->value (
                             sessionId+"/sshproxyuser",
                             QString()
-                        ).toString() );
+                        ).toString().trimmed() );
 
     proxyKey->setText(st.setting()->value (
                           sessionId+"/sshproxykeyfile",
                           QString()
-                      ).toString() );
+                      ).toString().trimmed() );
 
     proxyHost->setText(st.setting()->value (
                            sessionId+"/sshproxyhost",
                            QString()
-                       ).toString() );
+                       ).toString().trimmed() );
 
     proxyPort->setValue(st.setting()->value (
                             sessionId+"/sshproxyport",
@@ -588,6 +592,10 @@ void SessionWidget::readConfig()
                                      sessionId+"/sshproxyautologin",
                                      false
                                  ).toBool() );
+    cbProxyKrbLogin->setChecked(st.setting()->value (
+                                     sessionId+"/sshproxykrblogin",
+                                     false
+                                 ).toBool() );
 
     if(proxyHost->text().indexOf(":")!=-1)
     {
@@ -612,14 +620,14 @@ void SessionWidget::readConfig()
     QString
     command=st.setting()->value (
                 sessionId+"/command",
-                ( QVariant ) mainWindow->getDefaultCmd() ).toString();
+                ( QVariant ) mainWindow->getDefaultCmd() ).toString().trimmed();
 
     rdpOptions=st.setting()->value ( sessionId+"/rdpoptions",
-                                     ( QVariant ) "" ).toString();
+                                     ( QVariant ) "" ).toString().trimmed();
     rdpServer=st.setting()->value ( sessionId+"/rdpserver",
-                                    ( QVariant ) "" ).toString();
+                                    ( QVariant ) "" ).toString().trimmed();
     xdmcpServer=st.setting()->value ( sessionId+"/xdmcpserver",
-                                      ( QVariant ) "localhost" ).toString();
+                                      ( QVariant ) "localhost" ).toString().trimmed();
 #ifdef Q_OS_LINUX
     cbDirectRDP->setChecked(st.setting()->value (
                                 sessionId+"/directrdp",false ).toBool());
@@ -750,6 +758,7 @@ void SessionWidget::setDefaults()
     cbProxySamePass->setChecked(false);
     cbProxySameUser->setChecked(false);
     cbProxyAutologin->setChecked(false);
+    cbProxyKrbLogin->setChecked(false);
 
     QTimer::singleShot(1, this,SLOT(slot_proxySameLogin()));
     QTimer::singleShot(2, this,SLOT(slot_proxyType()));
@@ -763,16 +772,16 @@ void SessionWidget::saveSettings()
 
     X2goSettings st ( "sessions" );
     st.setting()->setValue ( sessionId+"/name",
-                             ( QVariant ) sessName->text() );
+                             ( QVariant ) sessName->text().trimmed() );
     st.setting()->setValue ( sessionId+"/icon",
                              ( QVariant ) sessIcon );
     st.setting()->setValue ( sessionId+"/host",
-                             ( QVariant ) server->text() );
+                             ( QVariant ) server->text().trimmed() );
     st.setting()->setValue ( sessionId+"/user",
-                             ( QVariant ) uname->text() );
+                             ( QVariant ) uname->text().trimmed() );
 
     st.setting()->setValue ( sessionId+"/key",
-                             ( QVariant ) key->text() );
+                             ( QVariant ) key->text().trimmed() );
 #ifdef Q_OS_LINUX
     st.setting()->setValue ( sessionId+"/rdpport",
                              ( QVariant ) rdpPort->value() );
@@ -792,16 +801,16 @@ void SessionWidget::saveSettings()
     if ( sessBox->currentIndex() < OTHER )
         command=sessBox->currentText();
     else
-        command=cmd->text();
+        command=cmd->text().trimmed();
     if ( sessBox->currentIndex() == RDP )
     {
         command="RDP";
-        rdpServer=cmd->text();
+        rdpServer=cmd->text().trimmed();
     }
     if ( sessBox->currentIndex() == XDMCP )
     {
         command="XDMCP";
-        xdmcpServer=cmd->text();
+        xdmcpServer=cmd->text().trimmed();
     }
     if ( sessBox->currentIndex() == SHADOW )
     {
@@ -826,7 +835,7 @@ void SessionWidget::saveSettings()
     if ( sessBox->currentIndex() ==APPLICATION )
     {
         rootless=true;
-        command=mainWindow->internAppName ( cmdCombo->lineEdit()->text() );
+        command=mainWindow->internAppName ( cmdCombo->lineEdit()->text().trimmed() );
     }
     if ( sessBox->currentIndex() == PUBLISHED)
         published=true;
@@ -857,13 +866,14 @@ void SessionWidget::saveSettings()
     {
         st.setting()->setValue ( sessionId+"/sshproxytype","SSH");
     }
-    st.setting()->setValue (sessionId+"/sshproxyuser",proxyLogin->text());
-    st.setting()->setValue (sessionId+"/sshproxykeyfile",proxyKey->text());
-    st.setting()->setValue (sessionId+"/sshproxyhost",proxyHost->text());
+    st.setting()->setValue (sessionId+"/sshproxyuser",proxyLogin->text().trimmed());
+    st.setting()->setValue (sessionId+"/sshproxykeyfile",proxyKey->text().trimmed());
+    st.setting()->setValue (sessionId+"/sshproxyhost",proxyHost->text().trimmed());
     st.setting()->setValue (sessionId+"/sshproxyport",proxyPort->value());
     st.setting()->setValue (sessionId+"/sshproxysamepass",cbProxySamePass->isChecked());
     st.setting()->setValue (sessionId+"/sshproxysameuser",cbProxySameUser->isChecked());
     st.setting()->setValue (sessionId+"/sshproxyautologin",cbProxyAutologin->isChecked());
+    st.setting()->setValue (sessionId+"/sshproxykrblogin",cbProxyKrbLogin->isChecked());
 
     st.setting()->sync();
 }
