@@ -32,6 +32,8 @@
 #endif
 #include <math.h>
 
+#include <QUuid>
+
 #ifndef Q_OS_WIN
 #include <sys/socket.h> /* for socket(), connect(), send(), and recv() */
 #include <arpa/inet.h>  /* for sockaddr_in and inet_addr() */
@@ -54,11 +56,102 @@
 
 static bool isLibSshInited=false;
 
+
+#ifdef Q_OS_WIN
+#include <QSettings>
+// parse known_hosts file from libssh and export keys in registry to use with plink.exe
+void SshMasterConnection::parseKnownHosts()
+{
+    QFile fl(mainWnd->getHomeDirectory()+"/ssh/known_hosts");
+    if (!fl.open(QFile::ReadOnly))
+        return;
+    QSettings settings("HKEY_CURRENT_USER\\Software\\SimonTatham\\PuTTY\\SshHostKeys",
+                       QSettings::NativeFormat);
+    while (!fl.atEnd())
+    {
+        QString line=fl.readLine();
+        QStringList parts=line.split(' ',QString::SkipEmptyParts);
+        if (parts.count()!=3)
+            continue;
+
+        //lines in known_hosts have format:
+        //[host]:port <ssh-rsa|ssh-dss> <key>
+        //we proceeding only lines from libssh - no hashed hostnames
+        //or patterns are allowed
+
+        QString type="unknown";
+        QString port="22";
+        if (parts[1]=="ssh-dss")
+            type="dss";
+        if (parts[1]=="ssh-rsa")
+            type="rsa2";
+
+
+        QStringList hostParts=parts[0].split(":",QString::SkipEmptyParts);
+        if (hostParts.count()>1)
+            port=hostParts[1];
+        hostParts[0].replace("[","");
+        hostParts[0].replace("]","");
+
+        QString keyName=type+"@"+port+":"+hostParts[0];
+
+        QByteArray bytes=QByteArray::fromBase64(parts[2].toAscii());
+        QStringList fields;
+
+        //key is a set of data fields:
+        //[size][data][size][data].....[size][data]
+
+        for (int i=0; i<bytes.count();)
+        {
+            int size=0;
+            //first 4 bytes are for size of data fild (big-endian)
+            for (int j=0; j<4; ++j)
+            {
+                size+=((uchar)(bytes[i])) * pow(256,3-j);
+                i++;
+            }
+            QByteArray data;
+            data=bytes.mid(i,size);
+            QString hex;
+
+            for (int j=0; j<data.count(); ++j)
+            {
+                QString byte;
+                byte.sprintf("%02x",(uchar)(data[j]));
+                hex+=byte;
+            }
+            //remove leading '0'
+            for (;;)
+            {
+                if (hex.length()==0)
+                    break;
+                if (hex[0]=='0')
+                    hex.remove(0,1);
+                else
+                    break;
+            }
+            hex="0x"+hex;
+            fields<<hex;
+            i+=size;
+        }
+        //first element is a type of key, we don't need it
+        fields.removeFirst();
+        settings.setValue(keyName,fields.join(","));
+#ifdef DEBUG
+        x2goDebug<<"writing key in registry: HKEY_CURRENT_USER\\Software\\SimonTatham\\PuTTY\\SshHostKeys"<<endl;
+        x2goDebug<<keyName<<"="<<fields.join(",")<<endl;
+#endif
+    }
+    settings.sync();
+}
+#endif
+
+
 SshMasterConnection::SshMasterConnection (QObject* parent, QString host, int port, bool acceptUnknownServers, QString user,
         QString pass, QString key, bool autologin, bool krblogin,
         bool useproxy, ProxyType type, QString proxyserver, quint16 proxyport,
         QString proxylogin, QString proxypassword, QString proxykey,
-        bool proxyautologin ) : QThread ( parent )
+        bool proxyautologin, bool proxyKrbLogin ) : QThread ( parent )
 {
 #if defined ( Q_OS_DARWIN )
     // Mac OS X provides only 512KB stack space for secondary threads.
@@ -87,16 +180,29 @@ SshMasterConnection::SshMasterConnection (QObject* parent, QString host, int por
     this->proxyport=proxyport;
     this->proxylogin=proxylogin;
     this->proxypassword=proxypassword;
+    this->proxyKrbLogin=proxyKrbLogin;
     reverseTunnel=false;
     mainWnd=(ONMainWindow*) parent;
     kerberos=krblogin;
+    challengeAuthVerificationCode=QString::null;
+    if(this->user==QString::null||this->user.length()<=0)
+    {
+#ifdef Q_OS_WIN
+        this->user=getenv("USERNAME");
+#else
+        this->user=getenv("USER");
+#endif
+    }
 #ifdef DEBUG
     if (kerberos)
+    {
         x2goDebug<<"starting ssh connection with kerberos authentication"<<endl;
+    }
     else
+    {
         x2goDebug<<"starting ssh connection without kerberos authentication"<<endl;
+    }
 #endif
-    kerberos=false;
 #ifdef DEBUG
     x2goDebug<<"SshMasterConnection, instance "<<this<<" created";
 #endif
@@ -107,7 +213,7 @@ SshMasterConnection::SshMasterConnection (QObject* parent, ONMainWindow* mwd, QS
         int remotePort, QString localHost, int localPort, SshProcess* creator,
         bool useproxy, ProxyType type, QString proxyserver, quint16 proxyport,
         QString proxylogin, QString proxypassword, QString proxykey,
-        bool proxyautologin, int localProxyPort) : QThread ( parent )
+        bool proxyautologin, bool proxyKrbLogin, int localProxyPort) : QThread ( parent )
 {
 #if defined ( Q_OS_DARWIN )
     setStackSize (sizeof (char) * 1024 * 1024 * 2);
@@ -131,6 +237,7 @@ SshMasterConnection::SshMasterConnection (QObject* parent, ONMainWindow* mwd, QS
     this->proxypassword=proxypassword;
     this->proxytype=type;
     this->proxyautologin=proxyautologin;
+    this->proxyKrbLogin=proxyKrbLogin;
     this->proxykey=proxykey;
     this->localProxyPort=localProxyPort;
     reverseTunnelLocalHost=localHost;
@@ -255,12 +362,16 @@ SshMasterConnection* SshMasterConnection::reverseTunnelConnection ( SshProcess*
     SshMasterConnection* con=new SshMasterConnection (this, mainWnd, host,port,acceptUnknownServers,user,pass,
             key,autologin, remotePort,localHost,
             localPort,creator, useproxy, proxytype, proxyserver, proxyport, proxylogin,
-            proxypassword, proxykey, proxyautologin, localProxyPort );
+            proxypassword, proxykey, proxyautologin, proxyKrbLogin, localProxyPort );
     con->kerberos=kerberos;
 
+    con->setVerficationCode(challengeAuthVerificationCode);
+
     connect ( con,SIGNAL ( ioErr ( SshProcess*,QString,QString ) ),this,SIGNAL ( ioErr ( SshProcess*,QString,QString ) ) );
     connect ( con,SIGNAL ( stdErr ( SshProcess*,QByteArray ) ),this,SIGNAL ( stdErr ( SshProcess*,QByteArray ) ) );
     connect ( con,SIGNAL ( reverseListenOk ( SshProcess* ) ), this, SIGNAL ( reverseListenOk ( SshProcess* ) ) );
+    connect ( con,SIGNAL ( needPassPhrase(SshMasterConnection*, bool)), this, SIGNAL (needPassPhrase(SshMasterConnection*, bool)));
+
     con->keyPhrase=keyPhrase;
     con->keyPhraseReady=true;
     con->start();
@@ -284,13 +395,13 @@ void SshMasterConnection::run()
     {
 
         sshProxy=new SshMasterConnection (0, proxyserver, proxyport,acceptUnknownServers,
-                                          proxylogin, proxypassword, proxykey, proxyautologin, kerberos, false);
+                                          proxylogin, proxypassword, proxykey, proxyautologin, proxyKrbLogin, false);
         connect ( sshProxy, SIGNAL ( connectionOk(QString) ), this, SLOT ( slotSshProxyConnectionOk() ) );
 
         connect ( sshProxy, SIGNAL ( serverAuthError ( int,QString,SshMasterConnection* ) ),this,
                   SLOT ( slotSshProxyServerAuthError ( int,QString, SshMasterConnection* ) ) );
-        connect ( sshProxy, SIGNAL ( needPassPhrase(SshMasterConnection*)),this,
-                  SIGNAL ( needPassPhrase(SshMasterConnection*)) );
+        connect ( sshProxy, SIGNAL ( needPassPhrase(SshMasterConnection*, bool)),this,
+                  SIGNAL ( needPassPhrase(SshMasterConnection*, bool)) );
         connect ( sshProxy, SIGNAL ( serverAuthAborted()),this,
                   SLOT ( slotSshProxyServerAuthAborted()) );
         connect ( sshProxy, SIGNAL ( userAuthError ( QString ) ),this,SLOT ( slotSshProxyUserAuthError ( QString ) ) );
@@ -359,6 +470,11 @@ void SshMasterConnection::run()
 
 #ifdef Q_OS_WIN
     ssh_options_set ( my_ssh_session, SSH_OPTIONS_SSH_DIR, (mainWnd->getHomeDirectory()+"/ssh").toAscii());
+    if (kerberos)
+    {
+        parseKnownHosts();
+    }
+
 #endif
     ssh_options_set(my_ssh_session, SSH_OPTIONS_LOG_VERBOSITY, &verbosity);
 
@@ -496,7 +612,13 @@ void SshMasterConnection::run()
         }
         QString err;
         if (!kerberos)
+        {
             err=ssh_get_error ( my_ssh_session );
+        }
+        else
+        {
+            err=sshProcErrString;
+        }
         QString message=tr ( "Authentication failed" );
 #ifdef DEBUG
         x2goDebug<<message<<" - "<<err;
@@ -695,20 +817,141 @@ int SshMasterConnection::serverAuth ( QString& errorMsg )
     return state;
 }
 
+void SshMasterConnection::setVerficationCode(QString code)
+{
+    challengeAuthVerificationCode=code;
+}
+
+
+bool SshMasterConnection::userChallengeAuth()
+{
+    int rez=ssh_userauth_kbdint(my_ssh_session, NULL, NULL);
+    int prompts;
+
+    switch( rez)
+    {
+    case SSH_AUTH_INFO:
+        prompts=ssh_userauth_kbdint_getnprompts(my_ssh_session);
+#ifdef DEBUG
+        x2goDebug<<"Have prompts: "<<prompts<<endl;
+#endif
+        if(prompts)
+        {
+            const char *prompt= ssh_userauth_kbdint_getprompt(my_ssh_session,0,NULL);
+#ifdef DEBUG
+            x2goDebug<<"Prompt[0]: |"<<prompt<<"|"<<endl;
+#endif
+            QString pr=prompt;
+            if(pr=="Password: ")
+            {
+#ifdef DEBUG
+                x2goDebug<<"Password request"<<endl;
+#endif
+                ssh_userauth_kbdint_setanswer(my_ssh_session,0,pass.toAscii());
+                return userChallengeAuth();
+            }
+            if(pr=="Verification code: ")
+            {
+#ifdef DEBUG
+                x2goDebug<<"Verification code request"<<endl;
+#endif
+
+                challengeAuthPasswordAccepted=true;
+                if(challengeAuthVerificationCode == QString::null)
+                {
+                    keyPhraseReady=false;
+                    emit needPassPhrase(this, true);
+                    for(;;)
+                    {
+                        bool ready=false;
+                        this->usleep(200);
+                        keyPhraseMutex.lock();
+                        if(keyPhraseReady)
+                            ready=true;
+                        keyPhraseMutex.unlock();
+                        if(ready)
+                            break;
+                    }
+                    challengeAuthVerificationCode=keyPhrase;
+                    if(challengeAuthVerificationCode==QString::null)
+                    {
+                        authErrors<<tr("Authentication failed");
+                        return false;
+                    }
+                }
+                ssh_userauth_kbdint_setanswer(my_ssh_session,0,challengeAuthVerificationCode.toAscii());
+                return userChallengeAuth();
+            }
+            QString err=ssh_get_error ( my_ssh_session );
+            authErrors<<err;
+
+            return false;
+        }
+        else
+        {
+            return userChallengeAuth();
+        }
+    case SSH_AUTH_SUCCESS:
+#ifdef DEBUG
+        x2goDebug<<"Challenge auth ok"<<endl;
+#endif
+        return true;
+    case SSH_AUTH_DENIED:
+        if(!challengeAuthPasswordAccepted )
+        {
+            QString err=ssh_get_error ( my_ssh_session );
+            authErrors<<err;
+            return false;
+        }
+        else
+        {
+            challengeAuthVerificationCode=QString::null;
+            //try with another verification code
+            return userChallengeAuth();
+        }
+    default:
+        QString err=ssh_get_error ( my_ssh_session );
+        authErrors<<err;
+
+        return false;
+    }
+    return false;
+
+}
+
 
 bool SshMasterConnection::userAuthWithPass()
 {
-    int rc = ssh_userauth_password ( my_ssh_session, NULL, pass.toAscii() );
-    if ( rc != SSH_AUTH_SUCCESS )
+    int method = ssh_userauth_list(my_ssh_session, NULL);
+
+    if (method& SSH_AUTH_METHOD_INTERACTIVE)
     {
-        QString err=ssh_get_error ( my_ssh_session );
-        authErrors<<err;
 #ifdef DEBUG
-        x2goDebug<<"userAuthWithPass failed:" <<err<<endl;
+        x2goDebug<<"Challenge authentication"<<endl;
 #endif
-        return false;
+        challengeAuthPasswordAccepted=false;
+        return userChallengeAuth();
     }
-    return true;
+
+    if (method & SSH_AUTH_METHOD_PASSWORD)
+    {
+#ifdef DEBUG
+        x2goDebug<<"Password authentication"<<endl;
+#endif
+        int rc = ssh_userauth_password ( my_ssh_session, NULL, pass.toAscii() );
+        if ( rc != SSH_AUTH_SUCCESS )
+        {
+            QString err=ssh_get_error ( my_ssh_session );
+            authErrors<<err;
+#ifdef DEBUG
+            x2goDebug<<"userAuthWithPass failed:" <<err<<endl;
+#endif
+            return false;
+        }
+        return true;
+    }
+
+    return false;
 }
 
 
@@ -721,7 +964,7 @@ bool SshMasterConnection::userAuthAuto()
         if(!reverseTunnel)
         {
             keyPhraseReady=false;
-            emit needPassPhrase(this);
+            emit needPassPhrase(this, false);
             for(;;)
             {
                 bool ready=false;
@@ -796,7 +1039,7 @@ bool SshMasterConnection::userAuthWithKey()
         if(!reverseTunnel)
         {
             keyPhraseReady=false;
-            emit needPassPhrase(this);
+            emit needPassPhrase(this, false);
             for(;;)
             {
                 bool ready=false;
@@ -870,9 +1113,77 @@ bool SshMasterConnection::userAuthWithKey()
     return true;
 }
 
+bool SshMasterConnection::userAuthKrb()
+{
+    QProcess ssh;
+    QString sshCmd;
+
+    QUuid uuid = QUuid::createUuid();
+    QString uuidStr = uuid.toString().mid(1, 36).toLower();
+
+    QString shcmd = " echo X2GODATABEGIN:" + uuidStr + "; whoami; echo X2GODATAEND:" + uuidStr ;
+
+#ifdef Q_OS_WIN
+    sshCmd="plink -batch "+user+"@"+host+" -P "+
+           QString::number(port)+ shcmd;
+#else
+    sshCmd="ssh -o GSSApiAuthentication=yes "+user+"@"+host+" -p "+
+           QString::number(port)+ " -o PasswordAuthentication=no "+shcmd;
+#endif
+
+#ifdef DEBUG
+    x2goDebug<<"starting ssh:" <<sshCmd<<endl;
+#endif
+    ssh.start(sshCmd);
+
+
+    if (!ssh.waitForStarted(5000))
+    {
+        sshProcErrString=ssh.errorString();
+        authErrors<<sshProcErrString;
+#ifdef DEBUG
+        x2goDebug<<"ssh start failed:" <<sshProcErrString<<endl;
+#endif
+        return false;
+    }
+    if (!ssh.waitForFinished(20000))
+    {
+        sshProcErrString=ssh.errorString();
+        authErrors<<tr("Failed to start SSH Client. Please check your installation and GSSApi configuration");
+        authErrors<<sshProcErrString;
+#ifdef DEBUG
+        x2goDebug<<"ssh not finished:" <<sshProcErrString<<endl;
+#endif
+
+        return false;
+    }
+    QString outp=ssh.readAllStandardOutput();
+    QString err=ssh.readAllStandardError();
+#ifdef DEBUG
+    x2goDebug<<"ssh exited\n";
+    x2goDebug<<"stdout - "<<outp<<endl;
+    x2goDebug<<"stderr - "<<err<<endl;
+    x2goDebug<<"code - "<<ssh.exitCode()<<", status - "<<ssh.exitStatus()<<endl;
+#endif
+
+    QString begin_marker = "X2GODATABEGIN:"+uuidStr+"\n";
+    QString end_marker = "X2GODATAEND:"+uuidStr+"\n";
+    int output_begin=outp.indexOf(begin_marker) + begin_marker.length();
+    int output_end=outp.indexOf(end_marker);
+    outp = outp.mid(output_begin, output_end-output_begin);
+    outp.replace("\n","");
+
+    if (ssh.exitCode() == 0 && ssh.exitStatus() == 0 && outp== user)
+        return true;
+    authErrors<<tr("Check your GSSApi configuration or choose another authentication method");
+    return false;
+}
+
 
 bool SshMasterConnection::userAuth()
 {
+    if (kerberos)
+        return userAuthKrb();
     if ( autologin && key=="" )
         if ( userAuthAuto() )
             return true;
@@ -903,7 +1214,7 @@ void SshMasterConnection::addChannelConnection ( SshProcess* creator, int sock,
 }
 
 
-void SshMasterConnection::addChannelConnection ( SshProcess* creator, QString cmd )
+void SshMasterConnection::addChannelConnection ( SshProcess* creator, QString uuid, QString cmd )
 {
 
     ChannelConnection con;
@@ -911,6 +1222,7 @@ void SshMasterConnection::addChannelConnection ( SshProcess* creator, QString cm
     con.sock=-1;
     con.creator=creator;
     con.command=cmd;
+    con.uuid=uuid;
 
     channelConnectionsMutex.lock();
     channelConnections<<con;
@@ -1407,7 +1719,8 @@ void SshMasterConnection::finalize ( int item )
         close ( tcpSocket );
     }
     SshProcess* proc=channelConnections[item].creator;
+    QString uuid=channelConnections[item].uuid;
     channelConnections.removeAt ( item );
-    emit channelClosed ( proc );
+    emit channelClosed ( proc, uuid );
 }
 
