@@ -20,14 +20,13 @@
 
 #include <libssh/libssh.h>
 #include <QObject>
-
+#include <QProcess>
 #ifndef Q_OS_WIN
 #include <netinet/in.h>
 #endif
 
 #include "sshmasterconnection.h"
 
-
 class SshProcess : public QObject
 {
     Q_OBJECT
@@ -70,17 +69,25 @@ private:
     QString abortString;
     bool tunnel;
     bool normalExited;
-
+//only to use with krb (until no GSSAPI support in libssh)
+    QProcess* proc;
+    QString procUuid;
+    bool execProcess;
+    bool tunnelOkEmited;
 
 private slots:
     void slotCheckNewConnection();
     void slotStdErr(SshProcess* creator, QByteArray data);
     void slotStdOut(SshProcess* creator, QByteArray data);
     void slotIOerr(SshProcess* creator,QString message, QString sshSessionErr);
-    void slotChannelClosed(SshProcess* creator);
+    void slotChannelClosed(SshProcess* creator, QString uuid);
     void slotReverseTunnelOk(SshProcess* creator);
     void slotCopyOk(SshProcess* creator);
     void slotCopyErr(SshProcess* creator,QString message, QString sshSessionErr);
+    //krb stuff
+    void slotSshProcFinished( int exitCode, QProcess::ExitStatus exitStatus);
+    void slotSshProcStdErr();
+    void slotSshProcStdOut();
 signals:
     void sshFinished ( bool result, QString output, int processId);
     void sshTunnelOk(int processId);
